#!/bin/bash
# List of parameters passed through environment
#* reason                       -- why this script was called, one of: pre-init connect disconnect reconnect
#* UNIQUE_IDENTIFIER            -- unique identifier for active profile (coming from Shimo)
#* VPNGATEWAY                   -- vpn gateway address (always present)
#* TUNDEV                       -- tunnel device (always present)
#* INTERNAL_IP4_ADDRESS         -- address (always present)
#* INTERNAL_IP4_MTU             -- mtu (often unset)
#* INTERNAL_IP4_NETMASK         -- netmask (often unset)
#* INTERNAL_IP4_NETMASKLEN      -- netmask length (often unset)
#* INTERNAL_IP4_NETADDR         -- address of network (only present if netmask is set)
#* INTERNAL_IP4_DNS             -- list of dns servers
#* INTERNAL_IP4_NBNS            -- list of wins servers
#* INTERNAL_IP6_ADDRESS         -- IPv6 address
#* INTERNAL_IP6_NETMASK         -- IPv6 netmask
#* INTERNAL_IP6_DNS             -- IPv6 list of dns servers
#* CISCO_DEF_DOMAIN             -- default domain name
#* CISCO_BANNER                 -- banner from server
#* CISCO_SPLIT_DNS              -- comma-separated list of domain names with split DNS
#* CISCO_SPLIT_INC              -- number of networks in split-network-list
#* CISCO_SPLIT_INC_%d_ADDR      -- network address
#* CISCO_SPLIT_INC_%d_MASK      -- subnet mask (for example: 255.255.255.0)
#* CISCO_SPLIT_INC_%d_MASKLEN   -- subnet masklen (for example: 24)
#* CISCO_SPLIT_INC_%d_PROTOCOL  -- protocol (often just 0)
#* CISCO_SPLIT_INC_%d_SPORT     -- source port (often just 0)
#* CISCO_SPLIT_INC_%d_DPORT     -- destination port (often just 0)
#* CISCO_IPV6_SPLIT_INC         -- number of networks in IPv6 split-network-list
#* CISCO_IPV6_SPLIT_INC_%d_ADDR -- IPv6 network address
#* CISCO_IPV6_SPLIT_INC_$%d_MASKLEN -- IPv6 subnet masklen

# FIXMEs:

# Section A: route handling

# 1) The 3 values CISCO_SPLIT_INC_%d_PROTOCOL/SPORT/DPORT are currently being ignored
#   In order to use them, we'll probably need os specific solutions
#   * Linux: iptables -t mangle -I PREROUTING <conditions> -j ROUTE --oif $TUNDEV
#       This would be an *alternative* to changing the routes (and thus 2) and 3)
#       shouldn't be relevant at all)
# 2) There are two different functions to set routes: generic routes and the
#   default route. Why isn't the defaultroute handled via the generic route case?
# 3) In the split tunnel case, all routes but the default route might get replaced
#   without getting restored later. We should explicitely check and save them just
#   like the defaultroute
# 4) Replies to a dhcp-server should never be sent into the tunnel

# Section B: Split DNS handling

# 1) Maybe dnsmasq can do something like that
# 2) Parse dns packets going out via tunnel and redirect them to original dns-server

#env | sort
#set -x

# =========== script (variable) setup ====================================

export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin

readonly SCRIPT_FILE=$0
readonly SCRIPT_LOG_FILE="${SCRIPT_FILE%%.*}.log"
readonly SCRIPT_PATH="${0%/*}"

readonly HOOKS_DIR=/etc/vpnc
readonly RESOLV_CONF_BACKUP=/var/run/vpnc/resolv.conf-backup

readonly DEFAULT_ROUTE_FILE="${SCRIPT_PATH}/${UNIQUE_IDENTIFIER}_defaultroute_${VPNGATEWAY}.old"

readonly OSVER="$(sw_vers | grep 'ProductVersion:' | grep -o '10\.[0-9]*')"

# replace commas with whitespaces in list of DNS servers (workaround)
if [ -n "${CISCO_SPLIT_DNS}" ]; then CISCO_SPLIT_DNS="${CISCO_SPLIT_DNS//,/ }"; fi

# some systems, eg. Darwin & FreeBSD, prune /var/run on boot
if [ ! -d "/var/run/vpnc" ]; then
	mkdir -p /var/run/vpnc
	[ -x /sbin/restorecon ] && /sbin/restorecon /var/run/vpnc
fi

logMessage()
{
    echo "$(date '+%Y-%m-%d %T') *Shimo: "${@} >> "${SCRIPT_LOG_FILE}"
}

print_variables()
{
    if [ -n "${UNIQUE_IDENTIFIER}" ]; then logMessage "UNIQUE_IDENTIFIER = ${UNIQUE_IDENTIFIER}"; fi
    if [ -n "${VPNGATEWAY}" ]; then logMessage "VPNGATEWAY = ${VPNGATEWAY}"; fi
    if [ -n "${TUNDEV}" ]; then logMessage "TUNDEV = ${TUNDEV}"; fi
    if [ -n "${INTERNAL_IP4_ADDRESS}" ]; then logMessage "INTERNAL_IP4_ADDRESS = ${INTERNAL_IP4_ADDRESS}"; fi
    if [ -n "${INTERNAL_IP4_NETMASK}" ]; then logMessage "INTERNAL_IP4_NETMASK = ${INTERNAL_IP4_NETMASK}"; fi
    if [ -n "${INTERNAL_IP4_NETMASKLEN}" ]; then logMessage "INTERNAL_IP4_NETMASKLEN = ${INTERNAL_IP4_NETMASKLEN}"; fi
    if [ -n "${INTERNAL_IP4_NETADDR}" ]; then logMessage "INTERNAL_IP4_NETADDR = ${INTERNAL_IP4_NETADDR}"; fi
    if [ -n "${INTERNAL_IP4_DNS}" ]; then logMessage "INTERNAL_IP4_DNS = ${INTERNAL_IP4_DNS}"; fi
    if [ -n "${INTERNAL_IP4_NBNS}" ]; then logMessage "INTERNAL_IP4_NBNS = ${INTERNAL_IP4_NBNS}"; fi
    if [ -n "${INTERNAL_IP6_ADDRESS}" ]; then logMessage "INTERNAL_IP6_ADDRESS = ${INTERNAL_IP6_ADDRESS}"; fi
    if [ -n "${INTERNAL_IP6_NETMASK}" ]; then logMessage "INTERNAL_IP6_NETMASK = ${INTERNAL_IP6_NETMASK}"; fi
    if [ -n "${INTERNAL_IP6_DNS}" ]; then logMessage "INTERNAL_IP6_DNS = ${INTERNAL_IP6_DNS}"; fi
    if [ -n "${CISCO_DEF_DOMAIN}" ]; then logMessage "CISCO_DEF_DOMAIN = ${CISCO_DEF_DOMAIN}"; fi
    if [ -n "${CISCO_SPLIT_DNS}" ]; then logMessage "CISCO_SPLIT_DNS = ${CISCO_SPLIT_DNS}"; fi
    logMessage "ARG_DEFAULT_ROUTE = ${ARG_DEFAULT_ROUTE}"
    logMessage "ARG_OVERWRITE_NETCONF = ${ARG_OVERWRITE_NETCONF}"
    logMessage "ARG_FLUSH_DNS_CACHE = ${ARG_FLUSH_DNS_CACHE}"
    logMessage "ARG_MODIFY_RESOLV_CONF = ${ARG_MODIFY_RESOLV_CONF}"
}

# =========== script hooks =================================================

run_hooks() {
	HOOK="$1"

	if [ -d ${HOOKS_DIR}/${HOOK}.d ]; then
	    for script in ${HOOKS_DIR}/${HOOK}.d/* ; do
		[ -f $script ] && . $script
	    done
	fi
}

# =========== tunnel interface handling ====================================

do_ifconfig() 
{
    if [ -n "${INTERNAL_IP4_MTU}" ]; then
        MTU=$INTERNAL_IP4_MTU
    fi

    if [ -z "${MTU}" ]; then
        MTU=1412
    fi

    # Point to point interface require a netmask of 255.255.255.255 on some systems
    ifconfig "${TUNDEV}" inet "${INTERNAL_IP4_ADDRESS}" "${INTERNAL_IP4_ADDRESS}" netmask 255.255.255.255 mtu ${MTU} up

    # set the route as specified by the remote server
    if [ -n "${INTERNAL_IP4_NETMASK}" ]; then
        set_network_route ${INTERNAL_IP4_NETADDR} ${INTERNAL_IP4_NETMASK} ${INTERNAL_IP4_NETMASKLEN}
    fi

    # If the netmask is provided, it contains the address _and_ netmask
    if [ -n "${INTERNAL_IP6_ADDRESS}" ] && [ -z "${INTERNAL_IP6_NETMASK}" ]; then
        INTERNAL_IP6_NETMASK="${INTERNAL_IP6_ADDRESS}/128"
    fi
    if [ -n "${INTERNAL_IP6_NETMASK}" ]; then
        # Unlike for Legacy IP, we don't specify the dest_address
        # here on *BSD. OpenBSD for one will refuse to accept
        # incoming packets to that address if we do.
        # OpenVPN does the same (gives dest_address for Legacy IP
        # but not for IPv6).
        ifconfig "${TUNDEV}" inet6 ${INTERNAL_IP6_NETMASK} mtu ${MTU} up
    fi
}

# =========== route handling ====================================

get_default_gw() 
{
    # isn't -n supposed to give --numeric output?
    # apperently not...
    # Get rid of lines containing IPv6 addresses (':')
    # Get rid of lines containing symbolic links ('link#N')
    netstat -r -n | awk '/:|link/ { next; } /^(default|0\.0\.0\.0)/ { print $2; }'
}

set_vpngateway_route() 
{
    route add -host "${VPNGATEWAY}" "`get_default_gw`"
}

del_vpngateway_route() 
{
    route delete -host "${VPNGATEWAY}" "`get_default_gw`"
}

set_default_route() 
{
    DEFAULTGW="`get_default_gw`"
    echo "${DEFAULTGW}" > "${DEFAULT_ROUTE_FILE}"
    route delete default "${DEFAULTGW}"
    route add default "${INTERNAL_IP4_ADDRESS}"
}

set_network_route() 
{    
    NETWORK="$1"
    NETMASK="$2"
    NETMASKLEN="$3"
    if [[ "${NETWORK}" == "${INTERNAL_IP4_ADDRESS}" ]]; then
        return
    fi
    
    del_network_route "${NETWORK}" "${NETMASK}" "${NETMASKLEN}"
    
    if [ -n "${NETWORK}" ] && [ -n "${NETMASK}" ]; then
        route add "${NETWORK}" -netmask "${NETMASK}" "${INTERNAL_IP4_ADDRESS}"
    elif [ -n "${NETWORK}" ]; then
        route add "${NETWORK}" "${INTERNAL_IP4_ADDRESS}"
    fi
}

reset_default_route() 
{
    if [ -s "${DEFAULT_ROUTE_FILE}" ]; then
        route delete default "`get_default_gw`"
        route add default `cat "${DEFAULT_ROUTE_FILE}"`
        rm -f -- "${DEFAULT_ROUTE_FILE}"
    fi
}

del_network_route() 
{
    # routes are deleted automatically on device shutdown
    return
}

set_ipv6_default_route() 
{
    route add -inet6 default "${INTERNAL_IP6_ADDRESS}"
}

set_ipv6_network_route() 
{
    NETWORK="$1"
    NETMASK="$2"
    route add -inet6 "${NETWORK}/${NETMASK}" "${INTERNAL_IP6_ADDRESS}"
}

reset_ipv6_default_route() 
{
    route delete -inet6 default "${INTERNAL_IP6_ADDRESS}"
}

del_ipv6_network_route() 
{
    NETWORK="$1"
    NETMASK="$2"
    route delete -inet6 "${NETWORK}/${NETMASK}" "${INTERNAL_IP6_ADDRESS}"
}

# =========== DNS handling ====================================

flushDNSCache()
{
    if ${ARG_FLUSH_DNS_CACHE} ; then
        case "${OSVER}" in
            * )
                set +e # "grep" will return error status (1) if no matches are found, so don't fail on individual errors
                hands_off_ps="$( ps -ax | grep HandsOffDaemon | grep -v grep.HandsOffDaemon )"
                set -e # We instruct bash that it CAN again fail on errors
                if [ "${hands_off_ps}" = "" ] ; then
                    if [ -f /usr/bin/killall ] ; then
                        /usr/bin/killall -HUP mDNSResponder
                        logMessage "Flushed the DNS Cache"
                    else
                        logMessage "/usr/bin/killall not present. Not flushing the DNS cache"
                    fi
                else
                    logMessage "Hands Off is running. Not flushing the DNS cache"
                fi
                ;;
        esac
    fi
}

modify_resolvconf()
{
    if ! "${ARG_MODIFY_RESOLV_CONF}" ; then
        return
    fi
    
	grep '^#@VPNC_GENERATED@' /etc/resolv.conf > /dev/null 2>&1 || cp -- /etc/resolv.conf "$RESOLV_CONF_BACKUP"
	NEW_RESOLVCONF="#@VPNC_GENERATED@ -- this file is generated by vpnc
# and will be overwritten by vpnc
# as long as the above mark is intact"

	# Don't change the original value of CISCO_DEF_DOMAIN we need it later
	CISCO_DEF_DOMAIN_TEMP="$CISCO_DEF_DOMAIN"
	# Don't step on INTERNAL_IP4_DNS value, use a temporary variable
	INTERNAL_IP4_DNS_TEMP="$INTERNAL_IP4_DNS"
	exec 6< "$RESOLV_CONF_BACKUP"
	while read LINE <&6 ; do
		case "$LINE" in
			nameserver*)
				if [ -n "$INTERNAL_IP4_DNS_TEMP" ]; then
					read ONE_NAMESERVER INTERNAL_IP4_DNS_TEMP <<-EOF
	$INTERNAL_IP4_DNS_TEMP
EOF
					LINE="nameserver $ONE_NAMESERVER"
				else
					LINE=""
				fi
				;;
			search*)
				if [ -n "$CISCO_DEF_DOMAIN_TEMP" ]; then
					LINE="$LINE $CISCO_DEF_DOMAIN_TEMP"
					CISCO_DEF_DOMAIN_TEMP=""
				fi
				;;
			domain*)
				if [ -n "$CISCO_DEF_DOMAIN_TEMP" ]; then
					LINE="domain $CISCO_DEF_DOMAIN_TEMP"
					CISCO_DEF_DOMAIN_TEMP=""
				fi
				;;
		esac
		NEW_RESOLVCONF="$NEW_RESOLVCONF
$LINE"
	done
	exec 6<&-

	for i in $INTERNAL_IP4_DNS_TEMP ; do
		NEW_RESOLVCONF="$NEW_RESOLVCONF
nameserver $i"
	done
	if [ -n "$CISCO_DEF_DOMAIN" ]; then
		NEW_RESOLVCONF="$NEW_RESOLVCONF
search $CISCO_DEF_DOMAIN"
	fi
	echo "$NEW_RESOLVCONF" > /etc/resolv.conf

    logMessage "Modified the DNS configuration in resolv.conf"
}

modify_dns() 
{
    PSID=$( (scutil | grep PrimaryService | sed -e 's/.*PrimaryService : //')<<-EOF
        open
          show State:/Network/Global/IPv4
        quit
EOF )

    OVERRIDE_PRIMARY=""
    if [ -n "${CISCO_SPLIT_INC}" ]; then
        if [ ${CISCO_SPLIT_INC} -lt 1 ]; then
            # Must override for correct default route
            # Cannot use multiple DNS matching in this case
            OVERRIDE_PRIMARY='d.add OverridePrimary # 1'
        fi
    fi
    # Uncomment the following if/fi pair to use multiple
    # DNS matching when available.  When multiple DNS matching
    # is present, anything reading the /etc/resolv.conf file
    # directly will probably not work as intended.
    if [ -z "$CISCO_DEF_DOMAIN" ]; then
        # Cannot use multiple DNS matching without a domain
        OVERRIDE_PRIMARY='d.add OverridePrimary # 1'
    fi

    if ! "${ARG_DEFAULT_ROUTE}" ; then
        NO_DEFAULT_ROUTE="#"
    fi
  
    if "${ARG_OVERWRITE_NETCONF}" ; then
        OVERRIDE_PRIMARY='d.add OverridePrimary # 1'
    fi
  
    # update IPv4 State: settings
    scutil >/dev/null 2>&1 <<-EOF
    open
        d.init
        d.add ServerAddresses * ${INTERNAL_IP4_DNS}
        set State:/Network/Service/${TUNDEV}/DNS
        d.init
        # next line overrides the default gateway and breaks split routing
        ${NO_DEFAULT_ROUTE}d.add Router ${INTERNAL_IP4_ADDRESS}
        d.add Addresses * ${INTERNAL_IP4_ADDRESS}
        d.add SubnetMasks * 255.255.255.255
        d.add InterfaceName ${TUNDEV}
        ${OVERRIDE_PRIMARY}
        set State:/Network/Service/${TUNDEV}/IPv4
        # copy proxy settings from default interface
        d.init
        get Setup:/Network/Service/${PSID}/Proxies
        set Setup:/Network/Service/${TUNDEV}/Proxies
    close
EOF

    # prepare search domains
    if [ -z "${CISCO_SPLIT_DNS}" ]; then
        SEARCH_DOMAINS=${CISCO_DEF_DOMAIN}
    elif [[ "${CISCO_SPLIT_DNS}" == *"${CISCO_DEF_DOMAIN}"* ]]; then
        SEARCH_DOMAINS=${CISCO_SPLIT_DNS}
    else
        SEARCH_DOMAINS="${CISCO_SPLIT_DNS} ${CISCO_DEF_DOMAIN}"
    fi

    if [ -z "$CISCO_DEF_DOMAIN" ]; then
        NO_DOMAIN_NAME="#"
    fi

    # set DNS settings via State:
    if [ -n "${SEARCH_DOMAINS}" ]; then
        scutil >/dev/null 2>&1 <<-EOF
        open
            get State:/Network/Service/${TUNDEV}/DNS
            ${NO_DOMAIN_NAME}d.add DomainName ${CISCO_DEF_DOMAIN}
            d.add SearchDomains * ${SEARCH_DOMAINS}
            d.add SupplementalMatchDomains * ${SEARCH_DOMAINS}
            set State:/Network/Service/${TUNDEV}/DNS
        close
EOF
        # in 10.8 and higher, ServerAddresses and SearchDomains must be set via the Setup: key in addition to the State: key
        if [ "${OSVER}" == "10.8" ] || [ "${OSVER}" == "10.9" ]; then
            scutil >/dev/null 2>&1 <<-EOF
            open
                # copy DNS info from State:
                d.init
                get State:/Network/Service/${TUNDEV}/DNS
                set Setup:/Network/Service/${TUNDEV}/DNS
            close
EOF
        fi
    fi

    # set SMB settings via State:
    if [ -n "${INTERNAL_IP4_NBNS}" ]; then
        scutil >/dev/null 2>&1 <<-EOF
        open
            # add SMB information
            d.init
            d.add WINSAddresses * ${INTERNAL_IP4_NBNS}
            set State:/Network/Service/${TUNDEV}/SMB
        close
EOF
    fi
    
    # now read all modified values for debug log    
    readonly NEW_IP4_STATE_CONFIG="$( scutil <<-EOF |
        open
        show State:/Network/Service/${TUNDEV}/IPv4
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"
    readonly NEW_DNS_SETUP_CONFIG="$( scutil <<-EOF |
        open
        show Setup:/Network/Service/${TUNDEV}/DNS
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"
    readonly NEW_DNS_STATE_CONFIG="$( scutil <<-EOF |
        open
        show State:/Network/Service/${TUNDEV}/DNS
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"
    readonly NEW_SMB_STATE_CONFIG="$( scutil <<-EOF |
        open
        show State:/Network/Service/${TUNDEV}/SMB
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"
    readonly NEW_IP4_GLOBAL_CONFIG="$( scutil <<-EOF |
        open
        show State:/Network/Global/IPv4
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"
    readonly NEW_DNS_GLOBAL_CONFIG="$( scutil <<-EOF |
        open
        show State:/Network/Global/DNS
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"
    readonly NEW_SMB_GLOBAL_CONFIG="$( scutil <<-EOF |
        open
        show State:/Network/Global/SMB
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"

    logMessage "DEBUG:"
    logMessage "DEBUG: Configurations as read back after changes:"
    logMessage "DEBUG: State:/.../IPv4 = ${NEW_IP4_STATE_CONFIG}"
    logMessage "DEBUG: State:/.../DNS = ${NEW_DNS_STATE_CONFIG}"
    logMessage "DEBUG: State:/.../SMB = ${NEW_SMB_STATE_CONFIG}"
    logMessage "DEBUG:"
    logMessage "DEBUG: Setup:/.../DNS = ${NEW_DNS_SETUP_CONFIG}"
    logMessage "DEBUG:"
    logMessage "DEBUG: State:/Network/Global/IPv4 = ${NEW_IP4_GLOBAL_CONFIG}"
    logMessage "DEBUG: State:/Network/Global/DNS = ${NEW_DNS_GLOBAL_CONFIG}"
    logMessage "DEBUG: State:/Network/Global/SMB = ${NEW_SMB_GLOBAL_CONFIG}"

    logMessage "Saved the DNS and SMB configurations for later use"
    
    flushDNSCache
}

restore_dns()
{
    scutil >/dev/null 2>&1 <<-EOF
    open
        # remove VPN DNS and IP
        remove State:/Network/Service/${TUNDEV}/IPv4
        remove State:/Network/Service/${TUNDEV}/DNS
        remove State:/Network/Service/${TUNDEV}/SMB
        remove Setup:/Network/Service/${TUNDEV}/Proxies
        remove Setup:/Network/Service/${TUNDEV}/DNS
    close
EOF
    
    flushDNSCache
    
    logMessage "Restored the DNS and SMB configurations"
}

restore_resolvconf()
{
    if ! "${ARG_MODIFY_RESOLV_CONF}" ; then
        return
    fi
	if [ ! -f "$RESOLV_CONF_BACKUP" ]; then
		return
	fi
	grep '^#@VPNC_GENERATED@' /etc/resolv.conf > /dev/null 2>&1 && cat "$RESOLV_CONF_BACKUP" > /etc/resolv.conf
	rm -f -- "$RESOLV_CONF_BACKUP"

    logMessage "Restored the DNS configuration in resolv.conf"
}

# ========= Toplevel state handling  =======================================

do_pre_init() 
{
    return
}

do_connect() 
{
    if [ -n "${CISCO_BANNER}" ]; then
        logMessage "Connect Banner:"
        logMessage "${CISCO_BANNER}" | while read LINE ; do logMessage "|" "$LINE" ; done
        logMessage "//END"
    fi

    set_vpngateway_route
    do_ifconfig
    if [ -n "${CISCO_SPLIT_INC}" ]; then
        i=0
        while [ $i -lt $CISCO_SPLIT_INC ] ; do
            eval NETWORK="\${CISCO_SPLIT_INC_${i}_ADDR}"
            eval NETMASK="\${CISCO_SPLIT_INC_${i}_MASK}"
            eval NETMASKLEN="\${CISCO_SPLIT_INC_${i}_MASKLEN}"
            if [ $NETWORK != "0.0.0.0" ]; then
                set_network_route "${NETWORK}" "${NETMASK}" "${NETMASKLEN}"
            else
                set_default_route
            fi
            i=`expr $i + 1`
        done
        for i in $INTERNAL_IP4_DNS ; do
            if ! echo "$i" | grep : >/dev/null; then
                set_network_route "$i" "255.255.255.255" "32"
            fi
        done
    elif [ -n "${INTERNAL_IP4_ADDRESS}" ] && "${ARG_DEFAULT_ROUTE}"; then
        set_default_route
    fi
    
    if [ -n "${CISCO_IPV6_SPLIT_INC}" ]; then
        i=0
        while [ $i -lt $CISCO_IPV6_SPLIT_INC ] ; do
            eval NETWORK="\${CISCO_IPV6_SPLIT_INC_${i}_ADDR}"
            eval NETMASKLEN="\${CISCO_IPV6_SPLIT_INC_${i}_MASKLEN}"
            if [ $NETMASKLEN -lt 128 ]; then
                set_ipv6_network_route "${NETWORK}" "${NETMASKLEN}"
            else
                set_ipv6_default_route
            fi
            i=`expr $i + 1`
        done
        for i in ${INTERNAL_IP4_DNS} ; do
            if echo "$i" | grep : >/dev/null; then
                set_ipv6_network_route "$i" "128"
            fi
        done
    elif [ -n "${INTERNAL_IP6_NETMASK}" -o -n "${INTERNAL_IP6_ADDRESS}" ] && "${ARG_DEFAULT_ROUTE}"; then
        set_ipv6_default_route
    fi

    if [ -n "${INTERNAL_IP4_DNS}" ]; then
        modify_resolvconf
        modify_dns
    fi
}

do_disconnect() 
{
    if [ -n "${CISCO_SPLIT_INC}" ]; then
        i=0
        while [ $i -lt $CISCO_SPLIT_INC ] ; do
            eval NETWORK="\${CISCO_SPLIT_INC_${i}_ADDR}"
            eval NETMASK="\${CISCO_SPLIT_INC_${i}_MASK}"
            eval NETMASKLEN="\${CISCO_SPLIT_INC_${i}_MASKLEN}"
            if [ $NETWORK != "0.0.0.0" ]; then
                # FIXME: This doesn't restore previously overwritten
                #        routes.
                del_network_route "${NETWORK}" "${NETMASK}" "${NETMASKLEN}"
            else
                reset_default_route
            fi
            i=`expr $i + 1`
        done
        for i in ${INTERNAL_IP4_DNS} ; do
            del_network_route "$i" "255.255.255.255" "32"
        done
    else
        reset_default_route
    fi
    
    if [ -n "${CISCO_IPV6_SPLIT_INC}" ]; then
        i=0
        while [ $i -lt $CISCO_IPV6_SPLIT_INC ] ; do
            eval NETWORK="\${CISCO_IPV6_SPLIT_INC_${i}_ADDR}"
            eval NETMASKLEN="\${CISCO_IPV6_SPLIT_INC_${i}_MASKLEN}"
            if [ $NETMASKLEN -eq 0 ]; then
                reset_ipv6_default_route
            else
                del_ipv6_network_route "${NETWORK}" "${NETMASKLEN}"
            fi
            i=`expr $i + 1`
        done
        for i in ${INTERNAL_IP6_DNS} ; do
            del_ipv6_network_route "$i" "128"
        done
    elif [ -n "${INTERNAL_IP6_NETMASK}" -o -n "${INTERNAL_IP6_ADDRESS}" ]; then
        reset_ipv6_default_route
    fi

    del_vpngateway_route

    if [ -n "${TUNDEV}" ]; then
        restore_resolvconf
        restore_dns
    fi
}

do_reset_network() 
{
    if [ -n "${TUNDEV}" ]; then
        restore_resolvconf
        restore_dns
    fi
    
    # remove all (10) tun/tap devices
    for i in {0..9}
    do
       ifconfig tun${i} delete
       ifconfig tap${i} delete
    done
    
    return
}

#### Main

if [ -z "${reason}" ]; then
    echo "this script must be called from vpnc" 1>&2
    exit 1
fi

# process parameters to script.
# set defaults, if not already set before
if [ -z ${ARG_OVERWRITE_NETCONF} ]; then
    ARG_OVERWRITE_NETCONF="false"
fi
if [ -z ${ARG_DEFAULT_ROUTE} ]; then
    ARG_DEFAULT_ROUTE="false"
fi
if [ -z ${ARG_FLUSH_DNS_CACHE} ]; then
    ARG_FLUSH_DNS_CACHE="false"
fi
if [ -z ${ARG_MODIFY_RESOLV_CONF} ]; then
    ARG_MODIFY_RESOLV_CONF="false"
fi

readonly ARG_OVERWRITE_NETCONF ARG_DEFAULT_ROUTE ARG_FLUSH_DNS_CACHE ARG_MODIFY_RESOLV_CONF

logMessage "${reason} starting..."

# print available parameters
print_variables

case "${reason}" in
    pre-init)
		run_hooks pre-init
        do_pre_init
        ;;
    connect)
		run_hooks connect
        do_connect
		run_hooks post-connect
        ;;
    disconnect)
		run_hooks disconnect
        do_disconnect
		run_hooks post-disconnect
        ;;
    reset-network)
        do_reset_network
        ;;
    reconnect)
		run_hooks reconnect
    	;;
    *)
        echo "unknown reason '${reason}'. Maybe vpnc-script is out of date" 1>&2
        exit 1
        ;;
esac

logMessage "${reason} completed successfully."

exit 0
